/*
** getethbyname() and getethbyaddr()
** These are not BSD functions.
** 
** Created June 2000 by Cyrus Patel <cyp@fb14.uni-mainz.de>
*/
#if 0 /* geteth*() is not BSD */

/* ------------------ begin ethers.h ---------------- */
#ifndef __ETHERS_H__
#define __ETHERS_H__

#if 0
  #include <net/if_ether.h>
#else
  #include <sys/packon.h>
  struct ether_addr /* this is declared in <net/if_ether.h> */
  {
    unsigned char ether_addr_octet[6];
  };
  #include <sys/packoff.h>
#endif

#include <sys/packon.h>
struct ether 
{
    char name[128];
    struct ether_addr addr;
};
#include <sys/packoff.h>

extern void setethent(void);
extern void endethent(void);
extern struct ether *getethent(void);
extern struct ether *getethbyname(const char *name);
extern struct ether *getethbyaddr(const struct ether_addr *addr);

#endif /* __ETHERS_H__ */
/* ------------------ end ethers.h ---------------- */

#define NDBRGEN_PAYLOAD        struct ether /* what we deal with internally*/
#define NDBRGEN_RETTYPE        struct ether  /* what we return (not ptr!) */
#define NDBRGEN_ETCFNCONF      "ETHERS"      /* entry in wattcp.cfg */
#define NDBRGEN_ENDENTFXNN     endethent     /* what to generate for endXXXent() */
#define NDBRGEN_GETENTFXNN     getethent     /* what to generate for getXXXent() */
#define NDBRGEN_SETENTFXNN     setethent     /* what to generate for setXXXent() */
#define NDBRGEN_SETCANSTAYOPEN 0  /* does setXXXent take a 'stayopen' arg? */
#define NDBRGEN_HAVEALIASES    0  /* no, we don't need alias support */

#include "dbentgen.h" /* generate all the necessary code */ 

/* ======================================================================= */

struct ether * getethbyname (const char *name)
{
  /* begin enumeration by passing NULL to enum_db_entries */
  struct ether *e = enum_db_entries((struct ether *)0);
  while (e)
  {
    if (strcmp(name, e->name)==0)
      break;
    e = enum_db_entries(e); /* get next */
  }
  /* note that getetherxxx functions do not set h_errno */
  return e;
}

struct ether * getethbyaddr (const struct ether_addr *addr)
{
  /* begin enumeration by passing NULL to enum_db_entries */
  struct ether *e = enum_db_entries((struct ether *)0);
  while (e)
  {
    if (memcmp(addr, &(e->addr), sizeof(*addr))==0)
      break;
    e = enum_db_entries(e); /* get next */
  }
  /* note that getetherxxx functions do not set h_errno */
  return e;
}

/* ======================================================================= */

/* convert a stringified ethernet address into the binary form.
*/
static int __atoeth(const char *str, struct ether_addr *addr)
{
  unsigned int pos = 0;
  while (*str)
  {    
    int val = 0;
    if (*str >= '0' && *str <= '9')
      val = (*str++ - '0');
    else if (*str >= 'A' && *str <= 'F')
      val = (*str++ - 'A')+10; 
    else  if (*str >= 'a' && *str <= 'f') 
      val = (*str++ - 'a')+10; 
    else 
      return -1;
    if (*str >= '0' && *str <= '9')
      val = (val << 4)+(*str++ - '0');
    else if (*str >= 'A' && *str <= 'F')
      val = (val << 4)+(*str++ - 'A')+10; 
    else  if (*str >= 'a' && *str <= 'f') 
      val = (val << 4)+(*str++ - 'a')+10; 
    if (*str != ':' && pos < 5)
      return -1;
    if (*str != '\0' && pos == 5)
      return -1;
    if (pos == 6)
      return -1;
    addr->ether_addr_octet[pos++] = (unsigned char)val;
    if (*str)
      str++;
  } 
  if (pos != 6)
    return -1;
  return 0;
}

/* ----------------------------------------------------------------------- */

/* This is our entry parser called by the auto-generated code to 
** parse a line in "/etc/ethers". 
*/
static NDBRGEN_PAYLOAD *parser_cb(const char *tokens[], int numtoks, 
                                  NDBRGEN_PAYLOAD *resP)
{
  NDBRGEN_PAYLOAD *e = (NDBRGEN_PAYLOAD *)0;
  if (numtoks == 2)
  {
    /* "0:0:70:41:42:43  somename" */
    if (strlen(tokens[1]) < (sizeof(e->name)-1))
    {
      struct ether_addr addr;
      if (__atoeth(tokens[0], &addr) == 0)
      {
        e = resP;
        memcpy (&(e->addr), &addr, sizeof(e->addr));
        strcpy (e->name, tokens[1]);
      }
    }
  }
  return e;
}

#endif /* geteth*() is not BSD */

/* ====================================================================== */

#if defined(TEST_PROG)
static void dump_ent(const char *label, const struct ether *e)
{
  if (!e)
    printf("%s: no data!\n", label);
  else
  {
    printf ("%s: e->name: '%s' e->addr: '%02X:%02X:%02X:%02X:%02X:%02X'\n",
            label, e->name,
            (int)(e->addr.ether_addr_octet[0]),
            (int)(e->addr.ether_addr_octet[1]),
            (int)(e->addr.ether_addr_octet[2]),
            (int)(e->addr.ether_addr_octet[3]),
            (int)(e->addr.ether_addr_octet[4]),
            (int)(e->addr.ether_addr_octet[5]) );
  }
  return;
}

int main (int argc, char *argv[])
{
  if (argc > 1)
  {
    struct ether_addr addr;
    if (strcmp(argv[1],"all")==0)
      argc = 1; /* fall through */
    else if (__atoeth(argv[1], &addr) == 0)
      dump_ent("byaddr", getethbyaddr( &addr ));
    else
      dump_ent("byname", getethbyname( argv[1] ));
  }
  if (argc == 1)
  {
    struct ether *e;
    while ((e = getethent()) != NULL)
      dump_ent("getent", e);
  }
  return (0);
}
#endif /* TEST_PROG */
