/*
** getnetbyname() and getnetbyaddr()
** 
** Created June 2000 by Cyrus Patel <cyp@fb14.uni-mainz.de>
*/
#include <netdb.h> /* h_errno */

struct my_netent {
    struct netent ne;
    char namebuf[32];
    char aliasbuf[64];
    char *aliaslist[5];
};

#define NDBRGEN_PAYLOAD        struct my_netent  /* what we deal with internally*/
#define NDBRGEN_RETTYPE        struct netent  /* what we return (not ptr!) */
#define NDBRGEN_ETCFNCONF      "NETWORKS"      /* entry in wattcp.cfg */
#define NDBRGEN_ENDENTFXNN     endnetent     /* what to generate for endXXXent() */
#define NDBRGEN_GETENTFXNN     getnetent     /* what to generate for getXXXent() */
#define NDBRGEN_SETENTFXNN     setnetent     /* what to generate for setXXXent() */
#define NDBRGEN_SETCANSTAYOPEN 1  /* does setXXXent take a 'stayopen' arg? */
#define NDBRGEN_HAVEALIASES    1  /* yes, we need alias support */

#include "dbentgen.h" /* generate all the necessary db access skeleton code */ 
#include <sys/socket.h> /* AF_INET */
#include <arpa/inet.h>  /* inet_network */

/* ======================================================================= */

NDBRGEN_RETTYPE * getnetbyname (const char *name)
{
  if (name)
  {
    /* begin enumeration by passing NULL to enum_db_entries */
    NDBRGEN_PAYLOAD *e = enum_db_entries((NDBRGEN_PAYLOAD *)0);
    while (e)
    {
      unsigned int i;
      if (strcmp(name, e->namebuf)==0)
        return &(e->ne);
      i = 0;
      while (e->aliaslist[i])
      {
        if (strcmp(name, e->aliaslist[i])==0)
          return &(e->ne);
        i++;
      }
      e = enum_db_entries(e); /* get next */
    }
  }
  h_errno = NO_DATA;
  return (NDBRGEN_RETTYPE *)0;
}

NDBRGEN_RETTYPE * getnetbyaddr (long net, int type)
{
  if (type == AF_INET)
  {
    /* begin enumeration by passing NULL to enum_db_entries */
    NDBRGEN_PAYLOAD *e = enum_db_entries((NDBRGEN_PAYLOAD *)0);
    while (e)
    {
      if (net == e->ne.n_net)
        return &(e->ne);
      e = enum_db_entries(e); /* get next */
    }
  }
  h_errno = NO_DATA;
  return (NDBRGEN_RETTYPE *)0;
}

/* ======================================================================= */

/* This is our entry parser called by the auto-generated code to 
** parse a line in "/etc/networks". 
*/
static NDBRGEN_PAYLOAD *parser_cb(const char *tokens[], int numtoks, 
                                  NDBRGEN_PAYLOAD *resP)
{
  NDBRGEN_PAYLOAD *e = (NDBRGEN_PAYLOAD *)0;
  if (numtoks >= 2)
  { 
    /* "arpanet  10  arpa" */
    if (strlen(tokens[0]) < (sizeof(e->namebuf)-1))
    {
      struct in_addr naddr; 
      naddr.s_addr = inet_network (tokens[1]);
      if (naddr.s_addr != 0 && naddr.s_addr != 0xfffffffful)
      {
        e = resP;
        e->ne.n_name     = strcpy(e->namebuf, tokens[0]);
        e->ne.n_addrtype = AF_INET;
        e->ne.n_net      = naddr.s_addr;
        e->ne.n_aliases  = &(e->aliaslist[0]);

        db_add_aliases( &tokens[2], numtoks-2, e->aliasbuf, sizeof(e->aliasbuf),
             &(e->aliaslist[0]), sizeof(e->aliaslist)/sizeof(e->aliaslist[0]));
      }
    }
  }
  return e;
}

/* ======================================================================= */

#if defined(TEST_PROG)
static void dump_ent(const char *label, const struct netent *e)
{
  if (!e)
    printf("%s: no data!\n", label);
  else
  {
    int i;
    printf ("%s: n_name: '%s' n_addrtype: '%d'  n_net: '%s'\n",
       label, e->n_name, e->n_addrtype, inet_ntoa(inet_makeaddr(e->n_net,0)) );
    for (i = 0; e->n_aliases && e->n_aliases[i]; i++)
       printf ("        n_aliases[%d]: '%s'\n", i, e->n_aliases[i]);
  }
  return;
}

int main (int argc, char *argv[])
{
  if (argc > 1)
  {
    if (strcmp(argv[1],"all")==0)
      argc = 1; /* fall through */
    else 
    {
      if (*argv[1] >= '0' && *argv[1] <= '9')
        dump_ent("byaddr", getnetbyaddr( inet_network(argv[1]), AF_INET ));
      else
        dump_ent("byname", getnetbyname( argv[1] ));
    }
  }
  if (argc == 1)
  {
    struct netent *e;
    while ((e = getnetent()) != NULL)
      dump_ent("getent", e);
  }
  return (0);
}
#endif /* TEST_PROG */
